@@ -911,6 +911,10 @@ extern GType g_fdo_notification_backend_get_type (void);
 extern GType g_gtk_notification_backend_get_type (void);
 #endif
 
+#ifdef HAVE_COCOA
+extern GType g_cocoa_notification_backend_get_type (void);
+#endif
+
 #ifdef G_PLATFORM_WIN32
 
 #include <windows.h>
@@ -946,16 +950,6 @@ _g_io_win32_get_module (void)
   return gio_dll;
 }
 
-#undef GIO_MODULE_DIR
-
-/* GIO_MODULE_DIR is used only in code called just once,
- * so no problem leaking this
- */
-#define GIO_MODULE_DIR \
-  g_build_filename (g_win32_get_package_installation_directory_of_module (gio_dll), \
-		    "lib/gio/modules", \
-		    NULL)
-
 #endif
 
 void
@@ -1016,13 +1010,45 @@ _g_io_modules_ensure_extension_points_registered (void)
   G_UNLOCK (registered_extensions);
 }
 
+static gchar *
+get_gio_module_dir (void)
+{
+  gchar *module_dir;
+
+  module_dir = g_strdup (g_getenv ("GIO_MODULE_DIR"));
+  if (module_dir == NULL)
+    {
+#ifdef G_OS_WIN32
+      gchar *install_dir;
+
+      install_dir = g_win32_get_package_installation_directory_of_module (gio_dll);
+#ifdef _MSC_VER
+      /* On Visual Studio builds we have all the libraries and binaries in bin
+       * so better load the gio modules from bin instead of lib
+       */
+      module_dir = g_build_filename (install_dir,
+                                     "bin", "gio", "modules",
+                                     NULL);
+#else
+      module_dir = g_build_filename (install_dir,
+                                     "lib", "gio", "modules",
+                                     NULL);
+#endif
+      g_free (install_dir);
+#else
+      module_dir = g_strdup (GIO_MODULE_DIR);
+#endif
+    }
+
+  return module_dir;
+}
+
 void
 _g_io_modules_ensure_loaded (void)
 {
   static gboolean loaded_dirs = FALSE;
   const char *module_path;
   GIOModuleScope *scope;
-  const gchar *module_dir;
 
   _g_io_modules_ensure_extension_points_registered ();
   
@@ -1030,6 +1056,8 @@ _g_io_modules_ensure_loaded (void)
 
   if (!loaded_dirs)
     {
+      gchar *module_dir;
+
       loaded_dirs = TRUE;
       scope = g_io_module_scope_new (G_IO_MODULE_SCOPE_BLOCK_DUPLICATES);
 
@@ -1051,11 +1079,10 @@ _g_io_modules_ensure_loaded (void)
 	}
 
       /* Then load the compiled in path */
-      module_dir = g_getenv ("GIO_MODULE_DIR");
-      if (module_dir == NULL)
-        module_dir = GIO_MODULE_DIR;
+      module_dir = get_gio_module_dir ();
 
       g_io_modules_scan_all_in_directory_with_scope (module_dir, scope);
+      g_free (module_dir);
 
       g_io_module_scope_free (scope);
 
@@ -1084,6 +1111,9 @@ _g_io_modules_ensure_loaded (void)
       g_type_ensure (g_fdo_notification_backend_get_type ());
       g_type_ensure (g_gtk_notification_backend_get_type ());
 #endif
+#ifdef HAVE_COCOA
+      g_type_ensure (g_cocoa_notification_backend_get_type ());
+#endif
 #ifdef G_OS_WIN32
       g_type_ensure (_g_winhttp_vfs_get_type ());
 #endif
