@@ -251,8 +251,10 @@ g_socket_service_is_active (GSocketService *service)
  * g_socket_service_start:
  * @service: a #GSocketService
  *
- * Starts the service, i.e. start accepting connections
- * from the added sockets when the mainloop runs.
+ * Restarts the service, i.e. start accepting connections
+ * from the added sockets when the mainloop runs. This only needs
+ * to be called after the service has been stopped from
+ * g_socket_service_stop().
  *
  * This call is thread-safe, so it may be called from a thread
  * handling an incoming client request.
@@ -283,6 +285,10 @@ g_socket_service_start (GSocketService *service)
  * close the listening sockets, call g_socket_listener_close(). (This
  * will happen automatically when the #GSocketService is finalized.)
  *
+ * This must be called before calling g_socket_listener_close() as
+ * the socket service will start accepting connections immediately
+ * when a new socket is added.
+ *
  * Since: 2.22
  */
 void
@@ -337,7 +343,7 @@ g_socket_service_class_init (GSocketServiceClass *class)
    * Since: 2.22
    */
   g_socket_service_incoming_signal =
-    g_signal_new ("incoming", G_TYPE_FROM_CLASS (class), G_SIGNAL_RUN_LAST,
+    g_signal_new (I_("incoming"), G_TYPE_FROM_CLASS (class), G_SIGNAL_RUN_LAST,
                   G_STRUCT_OFFSET (GSocketServiceClass, incoming),
                   g_signal_accumulator_true_handled, NULL,
                   NULL, G_TYPE_BOOLEAN,
@@ -401,6 +407,10 @@ g_socket_service_ready (GObject      *object,
  * New listeners can be added with e.g. g_socket_listener_add_address()
  * or g_socket_listener_add_inet_port().
  *
+ * New services are created active, there is no need to call
+ * g_socket_service_start(), unless g_socket_service_stop() has been
+ * called before.
+ *
  * Returns: a new #GSocketService.
  *
  * Since: 2.22
