@@ -13,13 +13,13 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this library; if not, see <http://www.gnu.org/licenses/>.
  */
 #include <glib.h>
 #include <stdlib.h>
+#ifdef G_OS_UNIX
 #include <unistd.h>
+#endif
 
 #define N_THREADS	8
 #define N_ALLOCS	50000
